from core.lib.time_lib import secondsFromMorning
from core.lib.date import AbsDateFromEpoch
from core.report.connection import ConnectionSearcher
from core.admin import admin_main
from core.config import getConfig
from core.user import user_main
from ibs_lib.ibs_csv import IBSngDialect
import shutil
import csv
import time, os
from  zipfile import ZipFile, ZIP_DEFLATED

def writeToFile(filepath, filename, start_time, end_time, cols, details_cols, **kw):
        conds = {"login_time_from":start_time,
              "login_time_from_unit":"gregorian",
               "login_time_to":end_time,
               "login_time_to_unit":"gregorian"}
        conds.update(kw)

        searcher = ConnectionSearcher(conds,
                                      admin_main.getAdminLoader().getAdminByID(0),
                                      "ADMIN")

        searcher.writeConnectionLogsToFile(filepath,
                                           cols,
                                           details_cols,
                                           "connection_log_id",
                                           False,
                                           "jalali"
                                           )

        if include_onlines:
            addOnlinesToFile(filepath)

        if do_compress_file:
            compressFile(filepath, filename)

        if kw.has_key("service") and kw["service"] == "internet" and add_user_phone:
            addUserPhoneToInternetFile(filepath)

def writeLogsForYesterday(file_folder, cols, details_cols, **kw):
    today = time.time() - secondsFromMorning()
    yesterday = today - 24*3600

    today_g = AbsDateFromEpoch(today).getGregorianDate()
    yesterday_g = AbsDateFromEpoch(yesterday).getGregorianDate()

    filename = getFilenameFromEpoch(yesterday)
    filepath = os.path.join(file_folder, filename)

    writeToFile(filepath, filename, yesterday_g, today_g, cols, details_cols, **kw)
    return filepath

def writeVoIPLogsForYesterday(file_folder, **kw):
    kw["service"] = "VoIP"

    return writeLogsForYesterday(file_folder, ["user_id", "username", "login_time_formatted", "logout_time_formatted", "caller_id", "called_number"], ["called_ip"], **kw)

def writeInternetLogsForYesterday(file_folder, **kw):
    kw["service"] = "Internet"

    return writeLogsForYesterday(file_folder, ["user_id", "username", "login_time_formatted", "logout_time_formatted", "remote_ip", "caller_id"], ["mac"], **kw)


def getFilenameFromEpoch(epoch_time):
    """
        Returns filename, for report in "epoch_time"
    """
    return  time.strftime("%Y-%m-%d.csv", time.localtime(epoch_time))

def compressFile(filepath, filename):
    zip_filepath = filepath.rsplit(".", 1)[0] + ".zip"
    zip_file = ZipFile(zip_filepath, "w", ZIP_DEFLATED)
    zip_file.write(filepath, filename)
    zip_file.close()
    os.remove(filepath)

def addOnlinesToFile(filepath):

    fd = open(filepath, "a+")
    online_users = user_main.getOnline().getOnlineUsers()
    logout_time_formatted = AbsDateFromEpoch(time.time() - secondsFromMorning() - 1).getDate("gregorian")

    for user_id, user_obj in online_users.iteritems():
         for instance in user_obj.getInstances().itervalues():
#   for user_id, user_obj in online_users.items():
#        for instance in user_obj.getInstances().values():
            if instance.getLoginTime() > 24*3600:
                if instance.isInternet():
                    row = [user_id,
                           user_obj.getUserAttrs()["normal_username"],
                           AbsDateFromEpoch(instance.getLoginTime()).getDate("gregorian"),
                           logout_time_formatted,
                           instance["attrs"].get("remote_ip", ""),
                           instance["attrs"].get("caller_id", ""),
                           instance["attrs"].get("mac", "")
                           ]

                    row = map(str, row)
                    fd.write(";".join(row)+"\r\n")
    fd.close()

def addUserPhoneToInternetFile(filepath):
    caller_id_idx = 5

    reader = csv.reader(open(filepath, "r"))
    writer = csv.writer(open(filepath+".temp", "w+"), dialect=IBSngDialect)

#    reader = csv.reader(open(filepath, "r"))
#    x = IBSngDialect
#    x.delimiter = ','
#    writer = csv.writer(open(filepath+".temp", "w+"), dialect=x)

    header = reader.next()
    writer.writerow(header)

    for row in reader:
        if row[caller_id_idx] == "":
            try:
                loaded_user = user_main.getUserPool().getUserByID(row[0])
                if loaded_user.getUserAttrs().userHasAttr("phone"):
                    row[caller_id_idx] = loaded_user.getUserAttrs()["phone"]
            except:
                pass

        writer.writerow(row)

    writer.close()
    reader.close()

    shutil.move(filepath + ".temp", filepath)


def edit_columns_wireless_adsl(filename):
    sep = ';'
    lines = []
    for line in open(filename).read().splitlines():
        parts = line.split(sep)
        ## user_id;username;login_time_formatted;logout_time_formatted;remote_ip;caller_id;mac
        parts.pop(0) ## user_id removed
        parts.append(parts.pop(0))
        lines.append(sep.join(parts))
    open(filename, 'w').write('\n'.join(lines))


do_compress_file = False
include_onlines = True
add_user_phone = True
folder_path = "/var/www/archive"

voip_conds = {"successful":"yes"}
dialup_conds = {"successful":"yes", "ras_desc":["E1"]}
wireless_conds = {"successful":"yes", "ras_desc":["Wireless"]}
adsl_conds = {"successful":"yes", "ras_desc":["ADSL"]}



writeVoIPLogsForYesterday(os.path.join(folder_path, "voip"))
## add source_ip and destination_ip (called_ip)

dialup_filepath = writeInternetLogsForYesterday(os.path.join(folder_path, "dialup"), **dialup_conds)

wireless_filepath = writeInternetLogsForYesterday(os.path.join(folder_path, "wireless"), **wireless_conds)
edit_columns_wireless_adsl(wireless_filepath)

adsl_filepath = writeInternetLogsForYesterday(os.path.join(folder_path, "adsl"), **adsl_conds)
edit_columns_wireless_adsl(adsl_filename)



